import string
import subprocess
import os

class UpstreamServer(object):
    
    def __init__(self,host,port = None):
        self.host = host
        self.port = port
    
    def __str__(self):
        d = dict(self.__dict__)
        d['sep'] = ':' if self.port else ''
        d['port'] = self.port or ''
        return 'server {host}{sep}{port};'.format(**d)

class TornadoUpstream(object):
    
    def __init__(self,upstream_name,upstream_server):
        self.name = upstream_name
        self.upstream_server = upstream_server
    
    def __str__(self):
        return '''
    upstream {name} {{
        {upstream_server}
    }}'''.format(**self.__dict__)

class MainUpstream(object):
    
    def __init__(\
        self,
        name,
        tornado_upstreams,
        n_replicas,
        alias = ''):
        
        self.name = name
        self.tornado_upstreams = tornado_upstreams
        self.n_replicas = n_replicas
        self.alias = alias
    
    @property
    def _servers_str(self):
        return string.join(map(str,self.tornado_upstreams),'\n\t')
    
    def __str__(self):
        return '''
    upstream {name} {{
        {servers}
        consistent_hash {n_replicas} {alias};
    }}'''.format(\
        name = self.name,
        servers = self._servers_str,
        n_replicas = self.n_replicas,
        alias = self.alias)

class ServerConfig(object):
    
    def __init__(self,port = 80,tornado_upstream = None,alias = ''):
        self.port = port
        self.tornado_upstream = tornado_upstream
        self.dir = os.path.dirname(os.path.abspath(__file__))
        self.alias = alias
    
    def __str__(self):
        alias = '''
        proxy_set_header X-Oliver-Balanced {alias};
        proxy_set_header X-Oliver-Balanced $http_x_oliver_balanced;
        '''.format(alias = self.alias) if self.alias else ''
        
        return '''
    server {{
        listen {port};
    
        error_page 502 =503 /down/down.json;
    
        # Allow file uploads
        client_max_body_size 50M;
        
        types {{
          application/json json;
        }}
        
        location / {{
            proxy_pass_header Server;
            proxy_set_header Host $http_host;
            proxy_redirect off;
            proxy_set_header X-Real-IP $remote_addr;
            proxy_set_header X-Scheme $scheme;
            proxy_headers_hash_bucket_size 128;
            proxy_headers_hash_max_size 1024;
            {alias}
            proxy_pass http://{tornado_upstream};
            proxy_next_upstream error http_502 http_503 http_404;
            proxy_buffering off;
        }}
        
        location /down {{
            alias {dir};
        }}
    }}'''.format(port = self.port, 
                 tornado_upstream = self.tornado_upstream.name,
                 dir = self.dir,
                 alias = alias)

class NginxConfig(object):
    
    def __init__(\
            self,
            tornado_upstreams,
            main_upstream,
            server_configs,
            path = None,
            alias = ''):
        
        print 'ALIAS:',alias
        self.tornado_upstreams = tornado_upstreams
        self.main_upstream = main_upstream
        self.main_upstream.alias = alias
        self.server_configs = server_configs
        for sc in self.server_configs:
            sc.alias = alias
        self.path = path
        self._p = None
        self.nginx_path = '/usr/local/nginx/sbin/nginx' 
    
    @property
    def _backends_str(self):
        return string.join(map(str,self.tornado_upstreams),'\n')
    
    @property
    def _servers_str(self):
        return string.join(map(str,self.server_configs),'\n')
    
    def _write(self):
        if not self.path:
            raise Exception('You must specify a path')
        with open(self.path,'w') as f:
            f.write(str(self))
            f.flush()
    
    def start(self):
        self._write()
        self._stop()
        self._p = subprocess.Popen(\
            [self.nginx_path,'-c',self.path],
            stdout=subprocess.PIPE)
        self._p.wait()
        return self._p.stdout.read()
    
    def _stop(self):
        p = subprocess.Popen([self.nginx_path,'-s','stop','-c',self.path])
        p.wait()
    
    def stop(self):
        self._stop()
        os.remove(self.path)
    
    def reload_conf(self):
        self._write()
        p = subprocess.Popen([self.nginx_path,'-c',self.path,'-s','reload'])
        p.wait()
    
    def __str__(self):
        return '''
worker_processes 1;

error_log /usr/local/nginx/logs/error.log debug;
pid /usr/local/nginx/logs/nginx.pid;

events {{
    worker_connections 1024;
    use epoll;
}}

http {{
    {tornado_backends}
    {main_upstream}

    include /usr/local/nginx/conf/mime.types;
    default_type application/octet-stream;

    access_log /usr/local/nginx/logs/access.log;

    keepalive_timeout 65;
    proxy_read_timeout 200;
    sendfile on;
    tcp_nopush on;
    tcp_nodelay on;
    gzip on;
    gzip_min_length 1000;
    gzip_proxied any;
    gzip_types text/plain text/html text/css text/xml
               text/javascript application/x-javascript
               application/xml application/atom+xml application/json;

    # Only retry if there was a communication error, not a timeout
    # on the Tornado server (to avoid propagating "queries of death"
    # to all frontends)
    #proxy_next_upstream error http_502 http_503 http_404;    
    {server_configs}
}}'''.format(tornado_backends = self._backends_str,
             main_upstream = self.main_upstream,
             server_configs = self._servers_str)
    