from cStringIO import StringIO
from journal import IdentityEncoder

class LockException(Exception): pass

# TODO: This could be a RedisType-derived class too
class Lock(object):
    
    def __init__(self,key,mutex_server):
        super(Lock,self).__init__()
        self.mutex_server = mutex_server
        self.key = key
    
    def __enter__(self):
        if not self.mutex_server.set(self.key,True,nx = True):
            raise LockException()
        return self
    
    def __exit__(self,t,value,traceback):
        del self.mutex_server[self.key]


class RedisType(object):
    
    def __init__(self,server,name,encoder = None):
        super(RedisType,self).__init__()
        self.server = server
        self.name = name
        self.encoder = encoder or IdentityEncoder
    
    def encode(self,inst):
        return self.encoder.encode(inst)
    
    def decode(self,s):
        return self.encoder.decode(StringIO(s))
    
#    def __del__(self):
#        del self.server[self.name]

    
class RedisHash(RedisType):
    
    def __init__(self,server,name,encoder = None):
        super(RedisHash,self).__init__(server,name,encoder)
    
    def items(self):
        i = self.server.hgetall(self.name)
        return [(k,self.decode(v)) for k,v in i.iteritems()]
    
    def keys(self):
        return self.server.hkeys(self.name)
    
    def __len__(self):
        return self.server.hlen(self.name)
    
    def __bool__(self):
        return bool(self.__len__())
    
    def __getitem__(self,key):
        v = self.server.hget(self.name,key)
        if not v:
            raise KeyError(key)
        return self.decode(v)
    
    def __setitem__(self,key,value):
        v = self.encode(value)
        return self.server.hset(self.name,key,v)
    
    def __delitem__(self,key):
        self.server.hdel(self.name,key)
    
    def __contains__(self,key):
        return self.server.hexists(self.name,key)
    
    def clear(self):
        del self.server[self.name]
    
    def decr(self,key):
        self.server.incrby(key,-1)
    
    def incr(self,key):
        self.server.incrby(key,1)

class RedisFIFOQueue(RedisType):
    
    def __init__(self,server,name,encoder = None):
        super(RedisFIFOQueue,self).__init__(server,name,encoder)
    
    def push(self,value):
        self.server.rpush(self.name,self.encode(value))
    
    def pop(self):
        raw = self.server.lpop(self.name)
        if not raw:
            return None
        return self.decode(raw)

class RedisSet(RedisType):
    
    def __init__(self,server,name,encoder = None):
        super(RedisSet,self).__init__(server,name,encoder)
    
    def add(self,*args):
        self.server.sadd(self.name,*self.encode(args))
    
    def members(self):
        return self.server.smembers(self.name)
    
    def pop(self):
        return self.server.spop(self.name)

class RedisCount(RedisType):
    
    def __init__(self,server,name,encoder = None):
        super(RedisCount,self).__init__(server,name,encoder)
    
    def __add__(self,n):
        self.server.incr(self.name,n)
        return self
    
    def __sub__(self,n):
        self.server.decr(self.name,n)
        return self
    
    def __int__(self):
        return self.count

    def __bool__(self):
        return bool(self.count)
    
    @property
    def count(self):
        try:
            return int(self.server[self.name])
        except KeyError:
            return 0
    
    def __call__(self,v = None):
        if v:
            self.server[self.name] = int(v)
            return
        
        return self.count