import os
from uuid import uuid4
from time import time,sleep
from random import shuffle,randint
from shutil import rmtree

from redis import Redis
from db import Oliver

class Test(object):
    
    def __init__(self,n_keys = 500,value_size_range_mb = (10,100)):
        super(Test,self).__init__()
        self.keys = [uuid4().hex for _ in xrange(n_keys)]
        self.low,self.high = value_size_range_mb
        self.path = os.path.join('/tmp/',uuid4().hex)
        os.makedirs(self.path)
    
    def _file_size(self):
        return randint(self.low * 1024 * 10,self.high * 1024 * 10)
    
    def _random_bytes(self):
        return os.urandom(self._file_size())
    
    def writes(self):
        raise NotImplemented()
    
    def _reads(self):
        raise NotImplemented()
    
    def reads(self):
        print 'reads'
        start = time()
        self._reads()
        print '{cls} took {n} seconds'.format(\
                            cls = self.__class__.__name__,n = time() - start)
        
        start = time()
        self._reads()
        print '{cls} took {n} seconds'.format(\
                            cls = self.__class__.__name__,n = time() - start)
        start = time()
        self._reads()
        print '{cls} took {n} seconds'.format(\
                            cls = self.__class__.__name__,n = time() - start)
        start = time()
        self._reads()
        print '{cls} took {n} seconds'.format(\
                            cls = self.__class__.__name__,n = time() - start)
        self._cleanup()
        rmtree(self.path)
    
    def _cleanup(self):
        pass
    
    def __call__(self):
        self.writes()
        self.reads()
        

class Files(Test):
    
    def __init__(self):
        super(Files,self).__init__()
        
    
    def writes(self):
        print 'writes'
        for key in self.keys:
            with open(os.path.join(self.path,key),'wb') as f:
                f.write(self._random_bytes())
    
    def _reads(self):
        sh = list(self.keys)
        shuffle(sh)
        for key in sh:
            with open(os.path.join(self.path,key),'rb') as f:
                v = f.read()

class Db(Test):
    
    def __init__(self):
        super(Db,self).__init__()
        self.db = Oliver(self.path,initial_size_bytes = 100 * 1000 * 1024)
        self.db.start()
    
    def writes(self):
        print 'writes'
        for key in self.keys:
            self.db[key] = self._random_bytes()
        
        while len(self.db) < len(self.keys):
            print 'waiting...'
            sleep(1)
    
    def _reads(self):
        sh = list(self.keys)
        shuffle(sh)
        for key in sh:
            v = self.db[key]
    
    def reads(self):
        print 'reads'
        
        for x in xrange(4):
            start = time()
            self._reads()
            print '{cls} took {n} seconds'.format(\
                                cls = self.__class__.__name__,n = time() - start)
        
        self._cleanup()
        rmtree(self.path)
    
    def _cleanup(self):
        self.db.close()


if __name__ == '__main__':
    #Files()()
    Db()()
    
        
    
    
            

        
