import unittest2
from address import Address

class AddressTest(unittest2.TestCase):
    
    def test_error_when_no_arguments(self):
        self.assertRaises(ValueError,lambda : Address())
    
    def test_error_when_port_specified_twice(self):
        self.assertRaises(ValueError,lambda : Address('host:80',80))
    
    def test_correct_host_and_port_when_specified_as_string(self):
        a = Address('host:8000')
        self.assertEqual('host',a.host)
        self.assertEqual(8000,a.port)
    
    def test_correct_host_and_port_when_specified_seperately(self):
        a = Address('host',8000)
        self.assertEqual('host',a.host)
        self.assertEqual(8000,a.port)
    
    def test_str_gives_correct_results_for_default_port(self):
        a = Address('host')
        self.assertEqual('host',str(a))
            
    def test_str_gives_correct_result_for_host_and_port(self):
        a = Address('host',80)
        self.assertEqual('host:80',str(a))
    
    def test_can_expand_into_host_and_port_for_host_only(self):
        a = Address('host')
        host,port = a
        self.assertEqual('host',host)
        self.assertEqual(None,port)
    
    def test_can_expand_into_host_and_port_for_host_and_port(self):
        a = Address('host:80')
        host,port = a
        self.assertEqual('host',host)
        self.assertEqual(80,port)
    
    def test_tuple(self):
        a = Address('host',80)
        self.assertEqual(('host',80),tuple(a))